# Description:
#   deploy repos to somewhere
#
# Configuration:
#   HUBOT_GITHUB_TOKEN (required)
#
# Commands:
#   hubot deploy <repository> to <env> - Creates deploy pull request
#
# Author:
#   naoya

GitHubApi = require 'github'
async     = require 'async'
_         = require 'underscore'

module.exports = (robot) ->
  config =
    user: 'naoya'
    branch:
      staging:
        from: 'master'
        to: 'deployment/staging'
        
      production:
        from: 'master'
        to: 'deployment/production'


  token = process.env.HUBOT_GITHUB_TOKEN
  if !token
    return robot.logger.error """
      deploy.coffee is not loaded due to missing configuration
      #{__filename}
    """

  robot.respond /deploy ([\w-\.]+) to (\w+)/i, (msg) ->
    github = new GitHubApi version: "3.0.0"
    github.authenticate type: "oauth", token: token

    repo = msg.match[1]
    env  = msg.match[2]

    if not config.branch[env]
      msg.send "No such environment: #{env}"
      return

    source_branch = config.branch[env].from
    deployment_branch = config.branch[env].to

    async.waterfall [
      (callback) ->
        msg.send "Creating Pull Request: #{source_branch} -> #{deployment_branch}"
        github.pullRequests.create
          user: config.user
          repo: repo
          title: "#{new Date().toYMD('.')} #{env} deployment by #{msg.message.user.name}"
          head: "#{config.user}:#{source_branch}"
          base: deployment_branch
        , (err, res) ->
          if err?
            callback err
          else
            callback null, { "response": res }

      (context, callback) ->
        github.pullRequests.getCommits
          user: config.user
          repo: repo
          number: context["response"].number
          per_page: 100
        , (err, res) ->
          if err?
            callback err
          else
            messages = _.map res, (object) ->
              message = object.commit.message.split("\n")[0]
              return "    #{object.author.login}: #{message}"
            msg.send "#{res.length} commits will be deployed: \n" + messages.join("\n")
            msg.send "continue manual merge #{context['response'].html_url}"
            callback null, context
    ], (err, res) ->
      if err?
        msg.send "An error occured while deploying #{repo}: \n" + err.message
